#
# EDOS testing framework
# Copyright (C) 2006 EDGE-IT
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#

#
# Author: Francois Dechelle <fdechelle@mandriva.com>
#
    
#
# HTTP test reporter
#

import os
import stat
#import httplib
import mimetools
import mimetypes
import cookielib
import urllib2
from edostestrunner.testreporter import TestReporter

def get_content_type(fd):
    return mimetypes.guess_type(fd.name)[0] or 'application/octet-stream'

class PostRequest:
    def __init__(self, server, port, url):
        self.__server__ = server
        self.__port__ = port
        self.__url__ = url
        self.__connection__ = httplib.HTTPConnection( self.__server__, self.__port__)
        self.__boundary__ = mimetools.choose_boundary()
        self.__headers__ = { 'Content-Type': 'multipart/form-data; boundary=%s' % self.__boundary__ }
        self.__body__ = ''

    def addValue(self,name,value):
        self.__body__ += '--%s\r\n' % self.__boundary__
        self.__body__ += 'Content-Disposition: form-data; name="%s"\r\n' % name
        self.__body__ += '\r\n'
        self.__body__ += '%s\r\n' % value

    def addFile(self,name,fd):
        self.__body__ += '--%s\r\n' % self.__boundary__
        self.__body__ += 'Content-Disposition: form-data; name="%s"; filename="%s"\r\n' % (name, fd.name)
        self.__body__ += 'Content-Type: %s\r\n' % get_content_type(fd)
        file_size = os.fstat(fd.fileno())[stat.ST_SIZE]
        self.__body__ += 'Content-Length: %d\r\n' % file_size
        self.__body__ += '\r\n'
        while True:
            data = fd.read()
            if not data:
                break
            self.__body__ += data.decode('iso8859-15')
        fd.close()
        self.__body__ +='\r\n'

    def post(self):
        self.__body__ += '--%s--\r\n' % self.__boundary__
        self.__body__ += '\r\n'
        print 'sending request on port %d to http://%s%s' % (self.__port__,self.__server__,self.__url__)
        self.__connection__.request( 'POST', self.__url__, self.__body__.encode('iso8859-15'), self.__headers__)
        response = self.__connection__.getresponse()
        data = response.read()
        self.__connection__.close()
        if response.status != httplib.OK:
            raise httplib.HTTPException( response.reason)
        return (response, data)


""" Reports a test using a HTTP POST request
"""
class HTTPTestReporter(TestReporter):
    def __init__(self):
        self.__cookieJar = cookielib.CookieJar(policy = cookielib.DefaultCookiePolicy())
        self.__opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.__cookieJar))

    def post( url, values, storeCookie = False):
        # choose content type
        contentType = ''

        data = urllib.urlencode( values)
        request = urllib2.Request( url, data)
        response = self.__opener.open(request)
        if storeCookie:
            self.__cookieJar.extract_cookies( response, request)
        return response

